/*
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999 - 2003 Sylvia Wong <sylvia dot wong at ieee.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/

#include "sucker.hh" 

#include <math.h>
#//include <iostream.h>
#include <qtable.h>
#include <qheader.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qmessagebox.h>
//#include <assert.h>

/********************************************************************
   class for handling projects
*/
Manager::Manager()
{
    pList.setAutoDelete( true );
}

Manager::~Manager()
{
  
}

int Manager::size() const
{
    return pList.count();
}

void Manager::add(QString n, int t, QString c)
{
    Project *np = new Project();
    //assert(np != 0);
    np->name = n;
    np->spent = t;
    np->category = c;
    pList.append(np);
}

void Manager::remove(int ind)
{    
    pList.remove(ind);
}

QString Manager::getName(int ind) 
{
    if(ind >= pList.count()) 
        return QString::null; 
    return  pList.at(ind)->name;
}
QString Manager::getCategory(int ind) 
{
    if(ind >= pList.count()) return ((QString) 0);    
    return pList.at(ind)->category;
}

void Manager::setCategory(int ind, QString s)
{
    if(ind < pList.count())
        pList.at(ind)->category = s;
}

int Manager::getTimeSpent(int ind) 
{
    if(ind >= pList.count()) return -1;
    return pList.at(ind)->spent;
}

void Manager::setTimeSpent(int ind, int t)
{
    if(ind < pList.count())
        pList.at(ind)->spent = t;
}

void Manager::addTimeSpent(int ind, int t)
{
    const int MS_IN_A_DAY = 24 * 60 * 60 * 1000;
    
    if(ind < pList.count()) {
        int nt = pList.at(ind)->spent + t;

	// restrict time to within 0 and 24 hours
        if(nt < 0) pList.at(ind)->spent = 0;
	else if(nt > MS_IN_A_DAY) pList.at(ind)->spent = MS_IN_A_DAY;
	else pList.at(ind)->spent = nt;
    }
}

void Manager::clearTimeSpent(int ind)
{
    if(ind < pList.count())
        pList.at(ind)->spent = 0;
}

int Manager::getIndex(QString s)
{
    for(int i = 0; i < pList.count(); i++)
        if( pList.at(i)->name == s )  return i;
    return -1;
}

/********************************************************************
   the main time tracker class
*/
Sucker::Sucker(QString idir, int isaveFreq, int iautostop,
	       QWidget *parent, const char *name)
    : dir(idir), REFRESHDELAY(60000), autosaveFreq(isaveFreq),
      QWidget(parent,name)
{
    running = false;
    f_projlist = dir + "projectlist";
    pttwidget = parent;

    task = new QTable( this );
    task->setNumCols(2);
    task->setLeftMargin(0);
    // task->setTopMargin(0);
    task->horizontalHeader()->setLabel (0, "Time" );
    task->horizontalHeader()->setLabel (1, "Project Name" );
#ifdef DESKTOP_VERSION
    task->setSelectionMode(QTable::SingleRow);
    task->setFocusStyle(QTable::FollowStyle);
    task->setReadOnly(true);
#else
    task->setSelectionMode(QTable::Single);
    //task->setFocusStyle(QTable::FollowStyle);
    //task->setReadOnly(true);
#endif
    task->setColumnStretchable (NAMECOL, true);

    // read project list from file
    //loadProjectFile();

    QFile f(f_projlist);
    if ( f.open(IO_ReadOnly) ) {
        QTextStream ts( &f );
        QString s, c, n;
	int pos, i;
        while ( !ts.eof() ) {
	    n = c = "";
            s = ts.readLine().stripWhiteSpace();
	    pos = s.find('^');
	    if(pos > 0) {
		for(i=0; i<pos; i++) c += s[i];
		n = s.remove(0, pos+1);
	    }
	    else{
		c = "";
		n = s;
	    }

	    manager.add(n,0,c);
	    task->setNumRows(manager.size());
	    task->setText(manager.size()-1,NAMECOL,n);
	}
        f.close();
    }

    fetchDayFile();
    for(int i=0; i < manager.size(); i++) {
	task->setText(i,TIMECOL,printTime(manager.getTimeSpent(i)));
    }
    readSessionFile();
    
    connect(task, SIGNAL(currentChanged(int,int)), SLOT(changeProj(int,int)));

    saveTimerId = startTimer(autosaveFreq * 60000);

    // end of day timer
    QDateTime dt = QDateTime::currentDateTime();
    QDateTime dt2 = QDateTime( (QDate::currentDate()).addDays(1), QTime() );
    dayTimerId = startTimer( dt.secsTo(dt2) * 1000 );

    // autostop timer
    if(iautostop != -1) {
	int h = iautostop / 60;
	int m = iautostop - h * 60;
	QDateTime dt3 = QDateTime( QDate::currentDate(), QTime(h,m) );
	int diff = dt.secsTo(dt3);
	if( diff > 0 )
	    autostopTimerId = startTimer( diff * 1000 );
	else 
	    autostopTimerId = startTimer( (diff + 24*60*60) * 1000 );
    }
}

Sucker::~Sucker()
{
}

QString Sucker::getProject() const
{ 
    if(cur == -1) return (QString) 0;
    return task->text(cur,NAMECOL);
}

QString Sucker::getCategory()
{
    if(cur == -1) return (QString) 0;
    int i = manager.getIndex(task->text(cur,NAMECOL));
    if(i < 0) 
	return "";
    else
	return manager.getCategory(i);
}

void Sucker::setCategory(QString nc) 
{
    manager.setCategory(manager.getIndex(task->text(cur,NAMECOL)), nc);
    saveProjectFile();
}

bool Sucker::isRunning() const
{ 
    return running; 
}

QString Sucker::getTime() 
{
    if(cur == -1) return (QString) 0;
    int t = manager.getTimeSpent( manager.getIndex(task->text(cur,NAMECOL)) );
    if(running) 
        t += mTime.elapsed();
    return printTime(t);
}

int Sucker::getIntTime() 
{
    if(cur == -1) return -1;
    int t = manager.getTimeSpent( manager.getIndex(task->text(cur,NAMECOL)) );
    if(running) 
        t += mTime.elapsed();
    return t/60000;
}

void Sucker::setTime(int min)
{
    bool r = running;
    if(r) running = false;

    manager.setTimeSpent( manager.getIndex(task->text(cur,NAMECOL)), min * 60000);
    task->setText(cur,TIMECOL,getTime());

    if(r) {
	running = true;
	mTime.start();    // clear current timer
    }
}

void Sucker::setSaveFreq(int min)
{
    if(min >= 5 && min <= 60) {
	autosaveFreq = min;
	killTimer(saveTimerId);
	saveTimerId = startTimer(autosaveFreq * 60000);
    }
}

int Sucker::getSaveFreq() const
{
    return autosaveFreq;
}

void Sucker::setAutostopTime(int min)
{
    if(min >= 0 && min < 24*60) {
	killTimer(autostopTimerId);
	QDateTime dt = QDateTime::currentDateTime();
	int h = min / 60;
	int m = min - h * 60;
	QDateTime dt2 (QDate::currentDate(), QTime(h,m));
	int diff = dt.secsTo(dt2);
	if( diff > 0 ) {
	    autostopTimerId = startTimer( diff * 1000 );
        // cout << "set timer at " << diff << "\n";
	}
	else {
	    autostopTimerId = startTimer( (diff + 24*60*60) * 1000 );
        // cout << "set timer at " << (diff + 24*60*60) << "\n";
	}
    }
}

void Sucker::saveForReport()
{
    if(running) 
	manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), 
 			     mTime.restart());
    saveDayFile("Save for report");
}

QString Sucker::printTime(int t) const
{
    t /= 60000;
    int h = t / 60;
    int m = t - h * 60;

    QString s;
    s.sprintf("%2d:%02d", h, m);
    return s;
}

void Sucker::resizeEvent( QResizeEvent * )
{
    task->setGeometry(0,0,width(),height());
}

void Sucker::newTask(QString name, QString icat)  
{
    name = name.stripWhiteSpace();
    if(name.isEmpty()) return;
    icat = icat.stripWhiteSpace();

    QString text;

    // see whether it's already on the list
    int tr = task->numRows();
    for(int i=0; i<tr; i++) {
	text = task->text(i,NAMECOL);
	if(text == name)
	    return;
    }
    
    task->setNumRows(tr+1);
    task->setText(tr, NAMECOL, name);
    task->setText(tr, TIMECOL, " 0:00");

    manager.add(name,0,icat);

    if( manager.size() == 1 ) {  // first item added
	cur = 0;
	task->setCurrentCell(0,0);
    }

    // add entry to file
    QFile f(f_projlist);
    if ( f.open(IO_WriteOnly | IO_Append) ) { 
          QTextStream ts( &f );
	  if(!icat.isEmpty())  ts << icat.latin1() << "^";
	  ts << name.latin1() << "\n";
	  f.close();
    }
}
    

void Sucker::removeTask()
{
    if( cur == -1 ) return;
    if(running) {
	pause();
	emit killed();
    }
    manager.remove( manager.getIndex(task->text(cur,NAMECOL)));
    
    saveProjectFile();

    // set new current task
    int dump = cur;
    if(task->numRows() == 1) cur = -1;
    else {
	if(task->numRows() == (cur+1))  // last item is deleted
	    cur --;
	task->setCurrentCell(cur,0);
    }
    //task->removeRow(dump);
    loadProjectFile();
   
}


// parameter: cat - category to be removed
// all projects belonging to the category will be moved to Unfiled
// list of project displayed is not changed even if current category
// is displayed. (only future actions is affected)
void Sucker::removeCategory(QString cat)
{
    for(int i=0; i < manager.size(); i++) {
	if( manager.getCategory(i) == cat )
	    manager.setCategory(i, "");
    }
}


void Sucker::changeProj(int row, int col)
{
    //QTableItem* it = task->item( row, col  );
    //if ( it )
        
    if(cur == -1 || row == cur) return;

    bool r = running;
    if(r) pause();
    task->setCurrentCell(row,0);
    
    cur = row;
    if(r) start();
    emit changed();
}

void Sucker::increment(int t)
{
    if(cur == -1) return;
    manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), t * 60000);
    task->setText(cur,TIMECOL,getTime());
}

void Sucker::decrement(int t)
{
    if(cur == -1) return;
    manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), -(t * 60000) );
    task->setText(cur,TIMECOL,getTime());
}


void Sucker::changeView(QString vs)
{
    if(running) 
	manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), 
			     mTime.restart());  // save time spent and

    pause();

    // remove old items from listboxes
     task->setNumRows(0);

    // display new items
     int i;
    if(vs == "All") {
	task->setNumRows(manager.size());
	for(i=0; i < manager.size(); i++) {
	    task->setText(i,1,manager.getName(i));
	    task->setText(i,0,printTime(manager.getTimeSpent(i)) );
	}
    }
    else {
	if(vs == "Unfiled") {
	    for(i=0; i < manager.size(); i++) {
		if(manager.getCategory(i).isEmpty()) {
		    int j = task->numRows();
		    task->setNumRows(j+1);
		    task->setText(j,NAMECOL,manager.getName(i));
		    task->setText(j,TIMECOL,printTime(manager.getTimeSpent(i)) );
		}
	    }
	}
	else {
	    for(i=0; i < manager.size(); i++) {
		if(vs == manager.getCategory(i)) {
		    int j = task->numRows();
		    task->setNumRows(j+1);
		    task->setText(j,NAMECOL,manager.getName(i));
		    task->setText(j,TIMECOL,printTime(manager.getTimeSpent(i)) );
		}
	    }
	}
    }
    
    // set current item to first item
    if(task->numRows() == 0)
	cur = -1;
    else {
	cur = 0;
	task->setCurrentCell(0,0);
    }
}


/*
   timing routines
*/
void Sucker::timerEvent( QTimerEvent * te) 
{
    if(te->timerId() == minTimerId) {
	task->setText(cur,TIMECOL,getTime());
	emit timerTick();
    }
    else if(te->timerId() == saveTimerId) {
	if(running) 
	    manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), 
				 mTime.restart());  // save time spent and
	                                                // restart timer
	saveDayFile("Periodic save");
    }
    else if(te->timerId() == dayTimerId) {
	int temp = cur;
	killTimer(dayTimerId);
	dayTimerId = startTimer(24 * 60 * 60000);
	if(running) 
	    manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)), 
				 mTime.restart());
	saveDayFile("End of day", -1);
	for(int i=0; i < manager.size(); i++) {
	    manager.clearTimeSpent(i);
	    task->setText(i,TIMECOL," 0:00");
	}
	// current item get messed up by the clearing done above
	task->setCurrentCell(temp,0);
	cur = temp;
    }
    else if(te->timerId() == autostopTimerId) {
	if(running)  pause();
	emit autostopped();
	killTimer(autostopTimerId);
	autostopTimerId = startTimer(24 * 60 * 60000);
    }
}

void Sucker::start()
{
    if(!running && cur != -1) {
	running = true;
	mTime.start();
	minTimerId = startTimer(REFRESHDELAY);
    }
    else
        ;//cerr << "fail to start: " << running << " " << cur << endl;
}

void Sucker::pause()
{
    if(running) {
	running = false;
	killTimer(minTimerId);
	manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)),
			     mTime.elapsed());  // save the time spent
	saveDayFile("Periodic save");
    }
}

/*
  dayfiles
*/
QString Sucker::dayFileName(int dayoffset)
{
    QDate date = QDate::currentDate();
    if(dayoffset != 0)
	date = date.addDays(dayoffset);
    
    QString s;
    s.sprintf("%04d-%02d-%02d", date.year(), date.month(), date.day());
    return s;
}

void Sucker::saveDayFile(QString reason, int dayoffset)
{
    QString name;
    bool change = false;
    int i;
    
    if(dayoffset)
	name = dayFileName(dayoffset);
    else
	name = dayFileName();

    QFile f(dir + name);
    int t;

    if ( f.open(IO_WriteOnly) ) { 
          QTextStream ts( &f );
	  ts << "# TIMETRACKER log saved at ";
	  ts << (QDateTime::currentDateTime()).toString().ascii() << "\n";
	  ts << "# " << reason.latin1() << "\n";
	  for(i=0; i < manager.size(); i++) {
	      t = manager.getTimeSpent(i);
	      if( t >= 60000 ) { // > 1 min
		  ts << printTime(t).ascii() << " ";
		  if(!manager.getCategory(i).isEmpty())
		      ts << manager.getCategory(i).latin1() << "^";
		  ts << manager.getName(i).latin1() << "\n";
	      }
	  }
	  f.close();
    }
    else
	QMessageBox::warning(0, "KTimeTracker: Error",
	   QString("Cannot open dayfile") + name.ascii() + 
                   "for writing", "OK");
}

void Sucker::fetchDayFile()
{
    QString fn = dayFileName();
    QFile f(dir + fn);
    if ( f.open(IO_ReadOnly) ) {
	int h, m;
	int i, pos;
	bool ok;
	QString s, hs, ms;
	QTextStream ts( &f );
	while ( !ts.eof() ) {
	    hs = ms = s = "";
	    s = ts.readLine().stripWhiteSpace();
	    if(s[0] == '#')  continue;  // skip comment lines
	    
	    pos = s.find(':');
	    for(i=0; i<pos; i++) hs += s[i];
	    h = hs.toInt(&ok);
	    if(!ok) {
		QMessageBox::warning(0, "KTimeTracker: Error",
	            QString("Invalid line in day file\n") + s, "OK");
		continue;
	    }
	    
	    ms = s[pos+1];
	    ms += s[pos+2];
	    m = ms.toInt(&ok);
	    if(!ok) {
		QMessageBox::warning(0, "KTimeTracker: Error",
	            QString("Invalid line in day file\n") + s, "OK");
		continue;
	    }
	    
	    if( (pos = s.find(' ')) == -1) {
		QMessageBox::warning(0, "KTimeTracker: Error",
	            QString("Invalid line in day file\n") + s, "OK");
		continue;
	    }

	    s.remove(0, pos+1);
	    pos = s.find('^');
	    if(pos > 0)   s.remove(0, pos+1);  // remove category
	    if( (i = manager.getIndex(s)) != -1)
		manager.setTimeSpent(i, (h*60+m)*60000);
	}
	f.close();
    }
}


/*
   read session file - current project and size of timelist and tasklist
*/
void Sucker::readSessionFile()
{
    int w1, w2, c;
    QFile f(dir + "session");    
    if( f.exists() && f.open(IO_ReadOnly) ) {
	QTextStream ts( &f );
	ts.readLine();
	ts >> c;   // assume no error in reading !!
	f.close();
    }

    // set current project
    if( manager.size() != 0 ) {
	cur = c;
	task->setCurrentCell(c,0);
    }
    else
	cur = -1;
}
void Sucker::loadProjectFile()
{
    QFile f(f_projlist);
    if ( f.open(IO_ReadOnly) ) {
        QTextStream ts( &f );
        QString s, c, n;
        int pos, i;
        int row = 0;
        task->setNumRows(manager.size());
        while ( !ts.eof() ) {
            n = c = "";
            s = ts.readLine().stripWhiteSpace();
            pos = s.find('^');
            if(pos > 0) {
                for(i=0; i<pos; i++) c += s[i];
                n = s.remove(0, pos+1);
            }
            else{
                c = "";
                n = s;
            }
            
            //manager.add(n,0,c);
            task->setText(row,NAMECOL,n);
            ++row;
        }
        f.close();
    }
}

void Sucker::saveProjectFile()
{
    // save new project file
    QFile f(f_projlist);
    if(f.open(IO_WriteOnly) ) {
	QTextStream ts( &f );
	for(int i=0; i < manager.size(); i++) {
	    if(!manager.getCategory(i).isEmpty())
		ts << manager.getCategory(i).latin1() << "^";
	    ts << manager.getName(i).latin1() << "\n";
	}
	f.close();
    }
    else
	QMessageBox::warning(0, "KTimeTracker: Error",
             QString("Cannot save project file ") + f_projlist, "OK");
}


void Sucker::dying()  // clean up before dying
{
    // save the time spent
    if(running)
	manager.addTimeSpent(manager.getIndex(task->text(cur,NAMECOL)),
			     mTime.elapsed());
    saveDayFile("Normal X termination");

    // remove lock
    QFile lf(dir + "LOCK");
    lf.remove();

    // sav session
    QFile f(dir + "session");
    if ( f.open(IO_WriteOnly) ) {
	QTextStream ts( &f );
	ts << pttwidget->width() << " " << pttwidget->height() << "\n";
	ts << cur << "\n";    // index to current project
	f.close();
    }
}
