/*
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999-2003 Sylvia Wong <s dot wong at auckland.ac.nz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/

#ifndef SUCKER_H
#define SUCKER_H

#include <qwidget.h>
#include <qstring.h>
#include <qtable.h>
#include <qdatetime.h>
#include <qlist.h>


/********************************************************************
   class for handling projects
   remember to keep task listed in Manager in sync of what's on screen
*/
class  Project     // a single project
{
public:
    QString name;      // its name
    QString category;  // its category
    int spent;         // time spent in milliseconds
};

class Manager
{
    QList<Project> pList;    // list of projects
    
public:
    Manager();
    ~Manager();
    int size() const;             // no of projects
    void add(QString n, int t, QString c = "");
                                  // add a new project(name, time spent, cat)
                                  // if category = "", unfiled

    void remove(int in);           // remove task in

    // input: ind is the index into list and t is the time spent
    // all time in millisecond
    QString getName(int ind);     // 0 for failure
    QString getCategory(int ind);
    void setCategory(int ind, QString s);
    int getTimeSpent(int ind) ;    // -1 for failure
    void setTimeSpent(int ind, int t);
    void addTimeSpent(int ind, int t);
    void clearTimeSpent(int ind);
    
    int getIndex(QString s) ;
    // return index into list of project s, -1 if s is not a valid project
};
    
/******************************************************************
   Main class
*/
class Sucker : public QWidget 
{
    Q_OBJECT
public:
    Sucker(QString idir, int isaveFreq, int iautostop,
	   QWidget *parent=0, const char *name=0);
    ~Sucker();
    QString getProject() const;     // returns name of current project
    QString getCategory() ;    // returns cat of current project
    void setCategory(QString nc);   // set cat of current project
    
    bool isRunning() const;         // true if timer is running
    QString getTime() ;     // returns time spent on current project in
                           // HH:MM form
    void setTime(int min); // set time spent on current project, in mins  
    int getIntTime();      // get time on current project in minutes
    void setSaveFreq(int min);  // in minutes (5-60)
    int getSaveFreq() const;
    void setAutostopTime(int min);  // set autostop time (in min after midnite)
    
    void saveForReport();

public slots:
    void removeTask();    // remove current task
    void removeCategory(QString cat);  // remove category cat

    void newTask(QString name,  QString icat);
    // add new task named name, no task is added if name is empty or
    // is already on list

    void start();    // start timer
    void pause();    // pause timer
    void dying();    // do last minute clean ups

    void increment(int t);
    void decrement(int t);
    // increment/decrement time spent on current task by t minutes

    void changeView(QString vs);  // category to display is changed
    
signals:
    void changed();    // change projects
    void killed();     // running project killed 
    void timerTick();  // a tick
    void autostopped();    // running project is autostopped

protected:
    void resizeEvent( QResizeEvent * );
    void timerEvent( QTimerEvent * );

private:
    QString printTime(int t) const;   // t in ms, return HH:MM

    QString dayFileName(int dayoffset = 0);
    // return name of dayfile, eg 1999-07-16 for 16 Jul 99

    void saveDayFile(QString reason, int dayoffset = 0);
    // reason is added as comment to dayfile

    void fetchDayFile();
    // fill the manager with time from dayfile, useful if timetracker has
    // been run earlier on the day

    void readSessionFile();
    void saveProjectFile();
    void loadProjectFile();

    Manager manager;

    QTable *task;  // where the times and task names are displayed
    enum {TIMECOL, NAMECOL};  // columns in the table
    int cur;        // index of current project (of both manager and qtable)

    bool running;       // is timer running
    int minTimerId;     // minute timer id
    int saveTimerId;    // auto save timer id
    int dayTimerId;     // end of day timer
    int autosaveFreq;   // auto save frequency in minutes
    int autostopTimerId;  // time (in min after midnite) to stop automatically
    QTime mTime;
    QWidget *pttwidget; // pointer to top level widget

    QString dir;        // timetracker dir
    QString f_projlist; // project list file name

    const int REFRESHDELAY; //in ms, how often to update titlebar and listboxes
    
private slots:
    void changeProj(int,int);       // change current project
};

#endif

