/*
    PTimeTracker - Qt based time tracking program
    Copyright (C) 1999-2003 Happy <ljm AT soton.ac.uk>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/
/*
CHANGELOG:

CHANGED 2004-07-24 Lutz Rogowski
*/


#ifndef WEEPORT_H
#define WEEPORT_H

#include <qwidget.h>
#include <qdialog.h>
#include <qdatetime.h>
#include <qfont.h>
#include <qtextview.h>

#include <qlist.h>
#include <qstringlist.h>

class QListBox;
class QPopupMenu;
class QString;
class QSpinBox;
class QSimpleRichText;

class Weeinfo;
class Weeport;
class Weedate;
class Weedisplay;

typedef struct 
{
    // a simple data structure to store the info coming in from the file.
    // it has an integer array entry for each of the seven days in a week
    QString caty_name;
    QString proj_name;
    int     proj_spnt[7];
} PData;

typedef struct
{
    QString cname;
    QStringList pnames;
} MData;

class Weeport : public QWidget 
{
    // The main report class. The purposie of which is to generate a simple
    // report based on the information in the files. It controls what range
    // over which the files are examined and the reports made. The final 
    // report is printed out in a QMultiLineEdit just using simple text 
    // formatting. A simple mechanism to allow selection of items to
    // report is maintained using a QListBox
    Q_OBJECT

public:
    Weeport();
    // Constructor : initialises the layout of the widgets involved for
    // the report display. no destuctor is needed as only have QT objects
    // dynamically allocated. 
    bool isWanted(QString name);
    // checks to see if this is one of the items required int he report.

private slots:
    void updateFromMenu( int id );
    // this method calls the functions that are required to update the
    // report when the selected item is changed.
    void printReport( );
    // this function will print the current report to the currently
    // attached printer
    void thisWeek( );
    // generate a report for the current week
    void lastWeek( );
    // generate a report for last week
    void chooseWeek( );
    // generate a report for an arbitrary week 

private:
    QPopupMenu      *item;          // the menu to select projects from
    QDate            thedate;       // the current date examined

    QList <MData> mdata;         // the data stored in the item menu

    Weedisplay      *disp;          // display for report data
    Weedate         *wdate;         // widget to alter date in 
    Weeinfo         *stuff;         // reads and gives info from file
    
    void updateList();
    // The purpose of this function is to update the listbox of all
    // available projects. This is called whenever the week is changed
    QString toHrsMins(int howlong);
    // Takes an integer (in minutes) and returns a string of the form
    // hh:mm
    QString extractData();
    // Takes the data from the info class and updates the QMultiLineEdit
    // with the new report info.
    void updateFromDate( );
    void doMenus( );

};

class Weeinfo
{
    // A class that acts as a gobetween for the report class and the
    // files containing the data. As such, it reads the data from the
    // appropriate files and stores them internally (using a vector).
    // Information can then be extracted via the use of the public
    // members.
public:
    Weeinfo(QDate date);
    // constructor : for a given date, the appropriate files are
    // examined and the data is extracted and stored in the vector.
    ~Weeinfo();
    // destructor : get rid of the data stored in the vector
    
    int numProjects();
    // return the number fo projects in this data set
    QString getProjectName(int i);
    // return the name of the project contained in location i
    QString getCategoryName(int i);
    // return the category of the project in location i
    int  getProjectData(int i, int day);
    // get the project data for a given location and day

private:
    QList <PData> projs;          // vector to store the info in

    QString dateToFile(QDate date);
    // Given a date convert it into the format used by the file
    void addData(QString s, int day);
    // Add the data in the file at a given day.

};

class Weedate : public QDialog 
{
    // A class that displays a dialog allowing you to select
    // an arbitrary date. you can ebter the week, month, and year
    // using spin boxes.
public:
    Weedate( );
    // constructor : sets up all the components in the dialog box.
    // initialises the spinboxes to todays date.

    QDate getDate( );
    // return the date that has been set via the spin boxes
    
private:
    QSpinBox *day;    // spin box to store day
    QSpinBox *month;  // and month
    QSpinBox *year;   // and year
    
};

class Weedisplay : public QScrollView
{
    // widget that contains the report that is to be displayed.
    // it does this by using a rich text widget.
public:
    Weedisplay( QWidget * parent=0, const char * name=0 );
    // constructor : initialises the data and the background

    void setText( QString text, QFont fnt );
    // set the contents of the widget to text and formats it
    // using font fnt.
    //QSize sizeHint ();
    // returns the best possible size for the widget

protected:
    void paintEvent(QPaintEvent *event);
    // updates any widgets that need updating.

private:
    QSimpleRichText     *data;   // the rich text data
    QTextView     *dataPrint;   // the rich text data

};

#endif // WEEPORT_H

